@@ -6,13 +6,15 @@
 */
 
 #include "kateview_test.h"
-#include "moc_kateview_test.cpp"
 
+#include <KSyntaxHighlighting/Repository>
+#include <KSyntaxHighlighting/Theme>
 #include <katebuffer.h>
 #include <kateconfig.h>
 #include <katedocument.h>
 #include <kateview.h>
 #include <kateviewinternal.h>
+#include <ktexteditor/editor.h>
 #include <ktexteditor/message.h>
 #include <ktexteditor/movingcursor.h>
 
@@ -40,34 +42,73 @@ KateViewTest::~KateViewTest()
 {
 }
 
+void KateViewTest::testEditorWidget()
+{
+    QCOMPARE_EQ(View::fromEditorWidget(nullptr), nullptr);
+    KTextEditor::DocumentPrivate doc(false, false);
+
+    auto *const view1 = doc.createView(nullptr);
+    auto *const editorWidget1 = view1->editorWidget();
+    QVERIFY(editorWidget1);
+    QCOMPARE(editorWidget1->metaObject()->className(), "KateViewInternal");
+    QCOMPARE_EQ(View::fromEditorWidget(editorWidget1), view1);
+
+    auto *const view2 = doc.createView(nullptr);
+    auto *const editorWidget2 = view2->editorWidget();
+    QVERIFY(editorWidget2);
+    QCOMPARE(editorWidget2->metaObject()->className(), "KateViewInternal");
+    QCOMPARE_EQ(View::fromEditorWidget(editorWidget2), view2);
+
+    QCOMPARE_NE(editorWidget1, editorWidget2);
+    QCOMPARE_EQ(view1->editorWidget(), editorWidget1);
+    QCOMPARE_EQ(View::fromEditorWidget(nullptr), nullptr);
+}
+
+void KateViewTest::testCoordinatesToCursor_data()
+{
+    using Flags = KTextEditor::View::CoordinatesToCursorFlags;
+    QTest::addColumn<Flags>("flags");
+
+    QTest::newRow("ClosestCursorOutsideText") << Flags{};
+    QTest::newRow("InvalidCursorOutsideText") << Flags{KTextEditor::View::InvalidCursorOutsideText};
+}
+
 void KateViewTest::testCoordinatesToCursor()
 {
     KTextEditor::DocumentPrivate doc(false, false);
     doc.setText(QStringLiteral("Hi World!\nHi\n"));
 
+    constexpr KTextEditor::Cursor beginningOfFirstLine(0, 0);
+    constexpr KTextEditor::Cursor endOfFirstLine(0, 9);
+
     KTextEditor::View *view1 = static_cast<KTextEditor::View *>(doc.createView(nullptr));
     view1->resize(400, 300);
     view1->show();
 
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(0, 2))), KTextEditor::Cursor(0, 2));
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(1, 1))), KTextEditor::Cursor(1, 1));
+    QFETCH(const KTextEditor::View::CoordinatesToCursorFlags, flags);
+    const auto coordinatesToCursor = [view1, flags](QPoint coords) {
+        return view1->coordinatesToCursor(coords, flags);
+    };
+
+    QCOMPARE(coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(0, 2))), KTextEditor::Cursor(0, 2));
+    QCOMPARE(coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(1, 1))), KTextEditor::Cursor(1, 1));
     // behind end of line should give an invalid cursor
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(1, 5))), KTextEditor::Cursor::invalid());
+    QCOMPARE(coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(1, 5))), KTextEditor::Cursor::invalid());
     QCOMPARE(view1->cursorToCoordinate(KTextEditor::Cursor(3, 1)), QPoint(-1, -1));
 
     // check consistency between cursorToCoordinate(view->cursorPosition() and cursorPositionCoordinates()
     // random position
     view1->setCursorPosition(KTextEditor::Cursor(0, 3));
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorToCoordinate(view1->cursorPosition())), KTextEditor::Cursor(0, 3));
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorPositionCoordinates()), KTextEditor::Cursor(0, 3));
+    QCOMPARE(coordinatesToCursor(view1->cursorToCoordinate(view1->cursorPosition())), KTextEditor::Cursor(0, 3));
+    QCOMPARE(coordinatesToCursor(view1->cursorPositionCoordinates()), KTextEditor::Cursor(0, 3));
     // end of line
-    view1->setCursorPosition(KTextEditor::Cursor(0, 9));
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(0, 9))), KTextEditor::Cursor(0, 9));
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorPositionCoordinates()), KTextEditor::Cursor(0, 9));
+    view1->setCursorPosition(endOfFirstLine);
+    QCOMPARE(coordinatesToCursor(view1->cursorToCoordinate(endOfFirstLine)), endOfFirstLine);
+    QCOMPARE(coordinatesToCursor(view1->cursorPositionCoordinates()), endOfFirstLine);
     // empty line
     view1->setCursorPosition(KTextEditor::Cursor(2, 0));
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(2, 0))), KTextEditor::Cursor(2, 0));
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorPositionCoordinates()), KTextEditor::Cursor(2, 0));
+    QCOMPARE(coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(2, 0))), KTextEditor::Cursor(2, 0));
+    QCOMPARE(coordinatesToCursor(view1->cursorPositionCoordinates()), KTextEditor::Cursor(2, 0));
 
     // same test again, but with message widget on top visible
     KTextEditor::Message *message = new KTextEditor::Message(QStringLiteral("Jo World!"), KTextEditor::Message::Information);
@@ -76,11 +117,33 @@ void KateViewTest::testCoordinatesToCursor()
     // wait 500ms until show animation is finished, so the message widget is visible
     QTest::qWait(500);
 
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(0, 2))), KTextEditor::Cursor(0, 2));
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(1, 1))), KTextEditor::Cursor(1, 1));
+    QCOMPARE(coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(0, 2))), KTextEditor::Cursor(0, 2));
+    QCOMPARE(coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(1, 1))), KTextEditor::Cursor(1, 1));
     // behind end of line should give an invalid cursor
-    QCOMPARE(view1->coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(1, 5))), KTextEditor::Cursor::invalid());
+    QCOMPARE(coordinatesToCursor(view1->cursorToCoordinate(KTextEditor::Cursor(1, 5))), KTextEditor::Cursor::invalid());
     QCOMPARE(view1->cursorToCoordinate(KTextEditor::Cursor(3, 1)), QPoint(-1, -1));
+
+    const auto leftmostCoordinates = view1->cursorToCoordinate(beginningOfFirstLine);
+    const auto rightmostCoordinates = view1->cursorToCoordinate(endOfFirstLine);
+
+    const auto adjusted = [](QPoint coordinates, int xOffset) {
+        auto ret = coordinates;
+        ret.rx() += xOffset;
+        return ret;
+    };
+
+    const auto expectedCursorToTheLeftOfText = flags & KTextEditor::View::InvalidCursorOutsideText ? KTextEditor::Cursor::invalid() : beginningOfFirstLine;
+    const auto expectedCursorToTheRightOfText = flags & KTextEditor::View::InvalidCursorOutsideText ? KTextEditor::Cursor::invalid() : endOfFirstLine;
+
+    QCOMPARE(coordinatesToCursor(adjusted(leftmostCoordinates, 0)), beginningOfFirstLine);
+    QCOMPARE(coordinatesToCursor(adjusted(leftmostCoordinates, 1)), beginningOfFirstLine); // the width of a character should be greater than 2 pixels
+    QCOMPARE(coordinatesToCursor(adjusted(leftmostCoordinates, -1000)), expectedCursorToTheLeftOfText);
+    QCOMPARE(coordinatesToCursor(adjusted(leftmostCoordinates, -1)), expectedCursorToTheLeftOfText);
+
+    QCOMPARE(coordinatesToCursor(adjusted(rightmostCoordinates, 0)), endOfFirstLine);
+    QCOMPARE(coordinatesToCursor(adjusted(rightmostCoordinates, -1)), endOfFirstLine); // the width of a character should be greater than 2 pixels
+    QCOMPARE(coordinatesToCursor(adjusted(rightmostCoordinates, 500)), expectedCursorToTheRightOfText);
+    QCOMPARE(coordinatesToCursor(adjusted(rightmostCoordinates, 1)), expectedCursorToTheRightOfText);
 }
 
 void KateViewTest::testCursorToCoordinates()
@@ -171,19 +234,6 @@ void KateViewTest::testIndentBlockSelection()
     QCOMPARE(doc.text(), QStringLiteral("nY\nnYY\n"));
 }
 
-namespace
-{
-QWidget *findViewInternal(KTextEditor::View *view)
-{
-    for (QObject *child : view->children()) {
-        if (child->metaObject()->className() == QByteArrayLiteral("KateViewInternal")) {
-            return qobject_cast<QWidget *>(child);
-        }
-    }
-    return nullptr;
-}
-}
-
 void KateViewTest::testSelection()
 {
     // see also: https://bugs.kde.org/show_bug.cgi?id=277422
@@ -211,7 +261,7 @@ void KateViewTest::testSelection()
     view->resize(100, 200);
     view->show();
 
-    auto *internalView = findViewInternal(view);
+    auto *const internalView = view->editorWidget();
     QVERIFY(internalView);
 
     const QPoint afterA = view->cursorToCoordinate(Cursor(0, 1));
@@ -482,7 +532,7 @@ void KateViewTest::testDragAndDrop()
     view->show();
     view->resize(400, 300);
 
-    QWidget *internalView = findViewInternal(view);
+    auto *const internalView = view->editorWidget();
     QVERIFY(internalView);
 
     // select "line1\n"
@@ -690,9 +740,6 @@ void KateViewTest::testFindMatchingFoldingMarker()
                        "    fi\n"
                        "done\n"));
 
-    KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
-    KateViewInternal *viewInternal = view->getViewInternal();
-
     const auto ifvalue = doc.buffer().computeFoldings(1)[0].foldingRegion;
     const auto dovalue = doc.buffer().computeFoldings(0)[0].foldingRegion;
 
@@ -702,19 +749,19 @@ void KateViewTest::testFindMatchingFoldingMarker()
     const KTextEditor::Range firstIfMatching(1, 36, 1, 38);
 
     // first test the do folding marker with cursor above first position of the word.
-    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstDo.start(), dovalue, 2000), firstDoMatching);
+    QCOMPARE(doc.buffer().findMatchingFoldingMarker(firstDo.start(), dovalue, 2000), firstDoMatching);
     // with cursor above last position of the word
-    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstDo.end(), dovalue, 2000), firstDoMatching);
+    QCOMPARE(doc.buffer().findMatchingFoldingMarker(firstDo.end(), dovalue, 2000), firstDoMatching);
     // now to test the maxLines param.
-    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstDo.start(), dovalue, 2), KTextEditor::Range::invalid());
+    QCOMPARE(doc.buffer().findMatchingFoldingMarker(firstDo.start(), dovalue, 2), KTextEditor::Range::invalid());
 
     // it must work from end folding to start folding too.
-    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstDoMatching.start(), dovalue.sibling(), 2000), firstDo);
-    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstDoMatching.start(), dovalue.sibling(), 2), KTextEditor::Range::invalid());
+    QCOMPARE(doc.buffer().findMatchingFoldingMarker(firstDoMatching.start(), dovalue.sibling(), 2000), firstDo);
+    QCOMPARE(doc.buffer().findMatchingFoldingMarker(firstDoMatching.start(), dovalue.sibling(), 2), KTextEditor::Range::invalid());
 
     // folding in the same line
-    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstIf.start(), ifvalue, 2000), firstIfMatching);
-    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstIfMatching.start(), ifvalue.sibling(), 2000), firstIf);
+    QCOMPARE(doc.buffer().findMatchingFoldingMarker(firstIf.start(), ifvalue, 2000), firstIfMatching);
+    QCOMPARE(doc.buffer().findMatchingFoldingMarker(firstIfMatching.start(), ifvalue.sibling(), 2000), firstIf);
 }
 
 void KateViewTest::testUpdateFoldingMarkersHighlighting()
@@ -732,7 +779,7 @@ void KateViewTest::testUpdateFoldingMarkersHighlighting()
                        "done\n"));
 
     KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
-    KateViewInternal *viewInternal = view->getViewInternal();
+    auto viewInternal = view->getViewInternal();
 
     const KTextEditor::Cursor positionWithoutMarker(0, 4);
     const KTextEditor::Range firstDo(0, 16, 0, 18);
@@ -742,20 +789,17 @@ void KateViewTest::testUpdateFoldingMarkersHighlighting()
     KTextEditor::MovingRange *foldingMarkerEnd = viewInternal->m_fmEnd.get();
 
     // If the cursor is not above any folding marker, the highlighting range is invalid
-    view->editSetCursor(positionWithoutMarker);
-    viewInternal->updateFoldingMarkersHighlighting();
+    view->setCursorPosition(positionWithoutMarker);
     QCOMPARE(foldingMarkerStart->toRange(), KTextEditor::Range::invalid());
     QCOMPARE(foldingMarkerEnd->toRange(), KTextEditor::Range::invalid());
 
     // If the cursor is above a opening folding marker, the highlighting range is the range of both opening and end folding markers words
-    view->editSetCursor(firstDo.start());
-    viewInternal->updateFoldingMarkersHighlighting();
+    view->setCursorPosition(firstDo.start());
     QCOMPARE(foldingMarkerStart->toRange(), firstDo);
     QCOMPARE(foldingMarkerEnd->toRange(), firstDoMatching);
 
     // If the cursor is above a ending folding marker, then same rule above
-    view->editSetCursor(firstDoMatching.start());
-    viewInternal->updateFoldingMarkersHighlighting();
+    view->setCursorPosition(firstDoMatching.start());
     QCOMPARE(foldingMarkerStart->toRange(), firstDo);
     QCOMPARE(foldingMarkerEnd->toRange(), firstDoMatching);
 }
@@ -839,6 +883,42 @@ void KateViewTest::testSelectedTextFormats()
     verifyFormat(formats[1], 2, 1, /*hasBg=*/false, /*hasFg=*/true);
     verifyFormat(formats[2], 3, 3, /*hasBg=*/false, /*hasFg=*/true);
     verifyFormat(formats[3], 6, 51, /*hasBg=*/false, /*hasFg=*/true);
+
+    // Test with block selection
+
+    const auto &repository = KTextEditor::Editor::instance()->repository();
+    auto lightTheme = repository.defaultTheme(KSyntaxHighlighting::Repository::LightTheme);
+    view->setConfigValue(QStringLiteral("theme"), lightTheme.name());
+
+    doc.setText(QStringLiteral("* hello\n* hello\n* hello\n"));
+    view->show();
+    view->setBlockSelection(true);
+    view->setCursorPosition({1, 4});
+    view->setSelection({0, 2, 1, 4});
+
+    // this test will break if someone removes "selected color" format normal textstyle
+    const QColor normalSelColor = lightTheme.selectedTextColor(KSyntaxHighlighting::Theme::TextStyle::Normal);
+
+    {
+        const QTextLayout *layoutLine0 = view->textLayout({0, 0});
+
+        auto fmt = layoutLine0->formats()[0];
+        QVERIFY(fmt.start == 0 && fmt.length == 2);
+
+        fmt = layoutLine0->formats()[1];
+        QVERIFY(fmt.start == 2 && fmt.length == 2 && fmt.format.foreground() == normalSelColor);
+
+        fmt = layoutLine0->formats()[2];
+        QVERIFY(fmt.start == 4 && fmt.length == 3 && fmt.format.foreground() != normalSelColor);
+
+        const QTextLayout *layoutLine1 = view->textLayout({1, 0});
+        fmt = layoutLine1->formats()[1];
+        QVERIFY(fmt.start == 2 && fmt.length == 2 && fmt.format.foreground() == normalSelColor);
+
+        fmt = layoutLine1->formats()[2];
+        QVERIFY(fmt.start == 4 && fmt.length == 3 && fmt.format.foreground() != normalSelColor);
+    }
 }
 
 // kate: indent-mode cstyle; indent-width 4; replace-tabs on;
+#include "moc_kateview_test.cpp"
