@@ -13,22 +13,14 @@
 #ifndef _KATE_VIEW_INTERNAL_
 #define _KATE_VIEW_INTERNAL_
 
-#include <KSyntaxHighlighting/FoldingRegion>
-
-#include <ktexteditor/attribute.h>
 #include <ktexteditor/range.h>
 #include <ktexteditor/view.h>
 
 #include "inlinenotedata.h"
 #include "katetextcursor.h"
-#include "katetextline.h"
 
-#include <QDrag>
-#include <QElapsedTimer>
 #include <QPoint>
 #include <QPointer>
-#include <QSet>
-#include <QTime>
 #include <QTimer>
 #include <QWidget>
 
@@ -41,6 +33,13 @@ class MovingRange;
 class TextHintProvider;
 class DocumentPrivate;
 class ViewPrivate;
+class Attribute;
+typedef QExplicitlySharedDataPointer<Attribute> AttributePtr;
+}
+
+namespace KSyntaxHighlighting
+{
+class FoldingRegion;
 }
 
 class KateIconBorder;
@@ -59,6 +58,7 @@ class QScrollBar;
 class QScroller;
 class QScrollEvent;
 class QScrollPrepareEvent;
+class QDrag;
 
 class KateViewInternal final : public QWidget
 {
@@ -73,13 +73,11 @@ class KateViewInternal final : public QWidget
     friend class WrappingCursor;
     friend class CamelCursor;
     friend class KateAbstractInputMode;
-    friend class ::KateTextPreview;
     friend class KateViewTest;
 
 public:
     enum Bias {
         left = -1,
-        none = 0,
         right = 1
     };
 
@@ -139,10 +137,6 @@ public:
     {
         return m_startPos.line();
     }
-    int startX() const
-    {
-        return m_startX;
-    }
 
     KTextEditor::Cursor endPos() const;
     int endLine() const;
@@ -219,24 +213,12 @@ public:
 
     QPoint cursorToCoordinate(const KTextEditor::Cursor cursor, bool realCursor = true, bool includeBorder = true) const;
     // by default, works on coordinates of the whole widget, eg. offsetted by the border
-    KTextEditor::Cursor coordinatesToCursor(const QPoint &coord, bool includeBorder = true) const;
+    [[nodiscard]] KTextEditor::Cursor
+    coordinatesToCursor(QPoint coord, KTextEditor::View::CoordinatesToCursorFlags flags = {}, bool includeBorder = true) const;
     QPoint cursorCoordinates(bool includeBorder = true) const;
     KTextEditor::Cursor findMatchingBracket();
 
-    // exported for unit tests
-    KTEXTEDITOR_EXPORT KTextEditor::Range
-    findMatchingFoldingMarker(const KTextEditor::Cursor current_cursor_pos, const KSyntaxHighlighting::FoldingRegion foldingRegion, const int maxLines);
-    KTEXTEDITOR_EXPORT void updateFoldingMarkersHighlighting();
-
-    inline int getStartOffset(int direction, int offset, int length) const
-    {
-        return direction == 1 ? offset - length : offset;
-    }
-
-    inline int getEndOffset(int direction, int offset, int length) const
-    {
-        return direction == 1 ? offset : offset + length;
-    }
+    void updateFoldingMarkersHighlighting();
 
     KateIconBorder *iconBorder() const
     {
@@ -247,8 +229,6 @@ public:
 private:
     void fixDropEvent(QDropEvent *event);
 
-    static bool isAcceptableInput(const QKeyEvent *e);
-
 protected:
     void hideEvent(QHideEvent *e) override;
     void paintEvent(QPaintEvent *e) override;
@@ -293,7 +273,7 @@ private:
     void scrollPos(KTextEditor::Cursor &c, bool force = false, bool calledExternally = false, bool emitSignals = true);
     void scrollLines(int lines, bool sel);
 
-    KTextEditor::Attribute::Ptr attributeAt(const KTextEditor::Cursor position) const;
+    KTextEditor::AttributePtr attributeAt(const KTextEditor::Cursor position) const;
     int linesDisplayed() const;
 
     int lineToY(int viewLine) const;
@@ -457,7 +437,7 @@ private:
     // implementation details for KTextEditor::FlashTextInterface
     //
 public:
-    void flashChar(const KTextEditor::Cursor pos, KTextEditor::Attribute::Ptr attribute);
+    void flashChar(const KTextEditor::Cursor pos, KTextEditor::AttributePtr attribute);
     void showBracketMatchPreview();
     void hideBracketMatchPreview();
 
